import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NgbDate, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import * as FileSaver from 'file-saver';
import { IDataOptions } from '@syncfusion/ej2-angular-pivotview';
import { ExcelReportService } from 'src/app/service/excel-report.service';
import { MyTemplateModel, TemplateModel } from 'src/app/model/template.model';
import { AlertModalComponent } from '../alert-modal/alert-modal.component';
import { MyModuleModel, ModuleModel } from 'src/app/model/module.model';
import { ConfirmModalComponent } from '../confirm-modal/confirm-modal.component';

export interface ModalDetail {
    text: { cardHead: string, search: string[], tableHead: string[] }
}
@Component({
    selector: 'app-excel-list',
    templateUrl: './excel-list.component.html',
    styleUrls: ['./excel-list.component.scss']
})
export class ExcelListComponent implements OnInit {
    template: { data: TemplateModel[], filter: TemplateModel[], loading: boolean } = { data: [], filter: [], loading: false }
    module: { data: ModuleModel[], loading: boolean } = { data: [], loading: false }
    page = 1
    pageSize = 10
    searchBy = ''
    condition = ''
    searchValue = ''
    openTemplate: Map<string, boolean> = new Map<string, boolean>()
    bodyTemplate: {
        status: 'add' | 'edit',
        data: {
            templateId: string,
            tname: string,
            ename: string,
            tdesc: string,
            edesc: string,
            module: string
        }
    } = {
            status: 'add',
            data: {
                templateId: '',
                tname: '',
                ename: '',
                tdesc: '',
                edesc: '',
                module: ''
            }
        }
    constructor(private excelReportService: ExcelReportService,
        private modalService: NgbModal,
        private cdr: ChangeDetectorRef) {

    }
    ngOnInit(): void {
        this.getExcelList()
        this.getModuleList()
    }
    getExcelList() {
        this.template.loading = true
        this.excelReportService.getExcelList().subscribe(response => {
            this.template.data = response.map(e => new MyTemplateModel(e))
            this.template.filter = response.map(e => new MyTemplateModel(e))
            this.template.loading = false
        }, error => {
            this.template.loading = false
        })
    }
    getModuleList() {
        this.module.loading = true
        this.excelReportService.getModuleList().subscribe(response => {
            this.module.data = response.map(e => new MyModuleModel(e))
            this.module.loading = false
        }, error => {
            this.module.loading = false
        })
    }
    postTemplate() {

    }
    openAlertModal(message?: string) {
        const modalRef = this.modalService.open(AlertModalComponent, {
            centered: true,
            backdrop: 'static'
        })
        modalRef.componentInstance.message = message ? message : ""
        modalRef.result.then(result => {
        }, reason => {
        })
    }
    openAddGroupModal(targetModal: NgbModal, data?: TemplateModel) {
        if (data) {
            this.bodyTemplate.status = 'edit'
            this.bodyTemplate.data = {
                templateId: data.templateId,
                tname: data.tname,
                ename: data.ename,
                tdesc: data.tdesc,
                edesc: data.edesc,
                module: data.module
            }
        } else {
            this.bodyTemplate.status = 'add'
            this.bodyTemplate.data = {
                templateId: '',
                tname: '',
                ename: '',
                tdesc: '',
                edesc: '',
                module: ''
            }
        }

        const modalRef = this.modalService.open(targetModal, {
            centered: true,
            backdrop: 'static',
            size: 'lg'
        })
        modalRef.result.then(result => {
            const modalConfirmRef = this.modalService.open(ConfirmModalComponent, {
                centered: true,
                backdrop: 'static',
            })
            modalConfirmRef.componentInstance.message = 'คุณต้องการบันทึกข้อมูลหรือไม่'
            modalConfirmRef.result.then(result => {
                console.log(" 🐒 this.bodyTemplate:", this.bodyTemplate)
                this.excelReportService.postTemplate(this.bodyTemplate.data).subscribe((response: any) => {
                    this.openAlertModal(response.message)
                    if (response.success) {
                        this.getExcelList()
                        this.getModuleList()
                    }

                }, error => { this.openAlertModal(error.message) })
            }, reject => { })
        }, reject => { })

    }

    openAddChildModal(targetModal: NgbModal) {
        const modalRef = this.modalService.open(targetModal, {
            centered: true,
            backdrop: 'static',
            size: 'lg'
        })
        modalRef.result.then(result => {
            const modalConfirmRef = this.modalService.open(ConfirmModalComponent, {
                centered: true,
                backdrop: 'static',
            })
            modalConfirmRef.componentInstance.message = 'คุณต้องการบันทึกข้อมูลหรือไม่'
            modalConfirmRef.result.then(result => {
                console.log(" 🐒 this.bodyTemplate:", this.bodyTemplate)
            }, reject => { })
        }, reject => { })
    }


    templateListSearch() {
        if (!this.searchBy || !this.condition || !this.searchValue) {
            this.template.filter = this.template.data.map(e => new MyTemplateModel(e))
        }
        const conditionMap: { [key: string]: (a: any, b: any) => boolean } = {
            includes: (a, b) => (a || '').toString().toLowerCase().includes((b || '').toString().toLowerCase()),
            lt: (a, b) => parseFloat(a) < parseFloat(b),
            gt: (a, b) => parseFloat(a) > parseFloat(b),
            eq: (a, b) => a == b,
            lte: (a, b) => parseFloat(a) <= parseFloat(b),
            gte: (a, b) => parseFloat(a) >= parseFloat(b),
            neq: (a, b) => a != b,
        };

        const compareFn = conditionMap[this.condition];
        if (!compareFn) {
            this.template.filter = this.template.data.map(e => new MyTemplateModel(e))
        }


        this.template.filter = this.template.data.filter(item => {
            const value = (item as any)[this.searchBy];
            return compareFn(value, this.searchValue);
        }).map(e => new MyTemplateModel(e));
    }
}